/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>



#define VER         "0.1"
#define FTPVOYDATA  "VictoriaPeterson"  // who is her???
#define FTPVOYFILE  "FTPVoyager.ftp"
#define OUTFILE     "FTPVoyager-pwd.txt"
#define BUFFSZ      65535               // too big, but it's ok
#define READIT(x)   \
        if(fread(&b1, 1, 1, fd) != 1) read_err(); \
        fputs(x, fdout); \
        if(b1 == 0xff) { \
            if(fread(&b2, 2, 1, fd) != 1) read_err(); \
            if(fread(buff, b2, 1, fd) != 1) read_err(); \
            fwrite(buff, b2, 1, fdout); \
        } else if(b1) { \
            if(fread(buff, b1, 1, fd) != 1) read_err(); \
            fwrite(buff, b1, 1, fdout); \
        }

#ifndef u_char
    #define u_char  unsigned char
#endif
#ifndef u_short
    #define u_short unsigned short
#endif
#ifndef u_int
    #define u_int  unsigned int
#endif



void ftpvoyager_pwddec(u_char *password, u_int size, u_int *encstring);
void read_err(void);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd,
            *fdout;
    u_char  *buff,
            b1;
    u_short b2;
    u_int  b4;


    setbuf(stdout, NULL);

    fputs("\n"
        "FTP Voyager passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    fputs("Opening file "FTPVOYFILE"\n", stdout);
    fd = fopen(FTPVOYFILE, "rb");
    if(!fd) std_err();


    fputs("Creating file "OUTFILE" (it will contain all the retrieved data)\n", stdout);
    fdout = fopen(OUTFILE, "wb");
    if(!fdout) std_err();

    if(fseek(fd, 6, SEEK_SET) < 0) std_err();
    if(fread(&b2, 2, 1, fd) != 1) read_err();
    if(fseek(fd, b2, SEEK_CUR) < 0) std_err();

    buff = malloc(BUFFSZ);
    if(!buff) std_err();

    while(1) {
        if(fread(&b4, 4, 1, fd) != 1) break;
        if(b4 != 0x43) {
            fputs("\n\n#######_F_O_L_D_E_R_##########\n", fdout);
            if(fseek(fd, -2, SEEK_CUR) < 0) std_err();    // new folder
            fread(&b4, 4, 1, fd);
            if(b4 != 0x43) break;
        }

        fputs("\n---------------", fdout);
        READIT("\n- Title:       ");
        READIT("\n- Description: ");
        READIT("\n- Server:      ");
        READIT("\n- Path:        ");
        READIT("\n- Username:    ");

        if(fread(&b4, 4, 1, fd) != 1) read_err();
        if(b4 > BUFFSZ) {
            fputs("\n"
                "The length of this encoded password seems not supported by FTP Voyager so the file is probably corrupted\n"
                "\n    Press RETURN to exit\n", stdout);
            fgetc(stdin);
            exit(1);
        }
        if(fread(buff, b4, 1, fd) != 1) read_err();
        ftpvoyager_pwddec(buff, b4, (u_int *)FTPVOYDATA);
        fprintf(fdout, "\n- Password:    %s\n", buff);

        if(fseek(fd, 0x43, SEEK_CUR) < 0) std_err();
        READIT("- Extra:       ");
        if(fseek(fd, 0xf3, SEEK_CUR) < 0) std_err();
    }

    fclose(fd);
    fclose(fdout);
    fputs("\n"
        "Finished\n"
        "\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void ftpvoyager_pwddec(u_char *password, u_int size, u_int *encstring) {
    u_int          num,
                    pwd1,
                    pwd2,
                    tmp,
                    *pwd;
    int             i;
    u_char          *ptr;

    size >>= 3;
    ptr = password;
    pwd = (u_int *)password;
    while(size--) {
        pwd1 = pwd[0];
        pwd2 = pwd[1];
        tmp = 0xc6ef3720;
        for(i = 32; i; i--) {
            num = ((pwd1 >> 5) + encstring[3]) ^
                  ((pwd1 << 4) + encstring[2]);
            pwd2 -= (num ^ (tmp + pwd1));

            num = ((pwd2 >> 5) + encstring[1]) ^
                  ((pwd2 << 4) + encstring[0]);
            pwd1 -= (num ^ (tmp + pwd2));

            tmp += 0x61c88647;
        }
        *ptr++ = pwd1;
        pwd++;
        pwd++;
    }
    *ptr = 0;
}



void read_err(void) {
    fputs("\nError: read error, file is probably corrupted\n", stdout);
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


